/*delay before and after encoder bus frame; >500ns*/
#define DELAYSTART delay_us(50);
/*encoder bus clock half-period, =1/(2*Fclk); Fclk<1MHz*/
#define DELAYCLK delay_us(100);
/*frame length of commutator relays switch impulse*/
#define DELAYSWITCH delay_ms(50);
/*frame length of step drive relays switch impulse*/
#define DELAYSTEP delay_ms(10);
#define FPIN PINA
//#define _SBI(P,b) P|=(1<<b);
//#define _CBI(P,b) P&=(!(1<<b));
#define DPIN PINB 
#define SPIN PINC
#define SPORT PORTC
#define PPIN PINC
#define PPORT PORTC 
#define DPORT PORTC
#define TPORT PORTD
#define EPORT PORTD
#define EPIN  PINE
#define UPORT PORTE
#define RPORT PORTD
const unsigned char ORR=0,ORL=1; 
const unsigned char IF11=0b00000001,IF12=0b00000010,IF21=0b00000100,IF22=0b00001000,IF10=0b00010000,IF20=0b00100000,IF01=0b01000000,IF02=0b10000000;
const unsigned char IDD11=1,IDD12=2,IDD21=4,IDD22=8,IDU11=16,IDU12=32,IDU21=64,IDU22=128;
const unsigned char IS1=1,IS2=2,OS1=2;
const unsigned char OP1=5,IP1=8,IP2=16;
const unsigned char ODD=6,ODU=7;
const unsigned char OT1=4,OU1=4;
const unsigned char OE_CSn=5,OE_CLK=6,IE_DO=128;
/*PIN DEFINITION
//I* ==2^pin_number
//O* ==pin_number
xPIN can be defined as 
      "#define xPIN PINx"
xPORT can be defined as 
      "#define xPORT PORTx" 
or 
      "#define xPORT ddrx"
(on latter case, portx must be set to 0, and:
     high-level output is equal to high-impendance state of output,
     low-level output is equal to low-impendance state of output)
I* can be defined as 
       "unsigned char I*=1<<P;"
O* can be defined as
       "unsigned char O*=P;"
EPIN  encoder bus port
      IE_DO DO wire, to encoder
EPORT encoder bus port
      OE_CLK CLK wire to encoder 
      OE_CSn CSn wire to encoder
RPORT  drive relay wires port
       ORR   clockwise rotation output          high to  step
       ORL   contr-clockwise rotation output    high to  step
FPIN  Failure inputs port
      IF11    RFA_stat_L1 GPS                   low if failure
      IF12    RFA_stat_L2 GPS                   low if failure
      IF21    RFA_stat_L1 GLONASS               low if failure
      IF22    RFA_stat_L2 GLONASS               low if failure
      IF01    MCU_POC; PGO GPS                  low if failure
      IF02    MCU_POC; PGO GLONASS              low if failure
      IF10    LED_FAIL GPS                      low if failure
      IF20    LED_FAIL GLONASS                  low if failure
SPIN  start control port (same as SPORT)
      IS1   start from control panel            high if start
      IS2   start from control panel            high if start
SPORT start wire port (same as SPIN)
      OS1   start to control panel              high to start
PPORT low power mode wire port (same as PPIN)
      OS1   LPM to control panel                high to LPM on 
PPIN  low power mode control wire port (same as PPORT)
      IS1   LPM from control panel              high if LPM on
      IS2   LPM from control panel              high if LPM on
DPIN commutators indication inputs port
     IDD11  L1 NAPR GPS                         low if on
     IDD12  L1 NAPR GLONASS                     low if on
     IDD21  L2 NAPR GPS                         low if on
     IDD22  L2 NAPR GLONASS                     low if on
     IDU11  L1 KRUG GPS                         low if on
     IDU12  L1 KRUG GLONASS                     low if on
     IDU21  L2 KRUG GPS                         low if on
     IDU22  L2 KRUG GLONASS                     low if on
DPORT commutation control wire port
      ODD   NAPR                                high to switch
      ODU   KRUG                                high to switch
TPORT fan relay wire port
      AIN0, AIN1 must be used for temperature control
      or
         AIN1 must be used for temperature control (1.23V at change point)
         and comparator must be initialised for internal voltage use:
         ACSR=ACSR|(1<<ACBG);
      OT1   fan relay output                    high to on
UPORT USART/UART direction control wire port
      RXD*,TXD* must be used for UART bus
      OU1   UART direction control, to driver   high to transmit
ENCODER PROTOCOL NOTES
1.less significant bit are transmitted first.
2.transmittion errors aren`t checked.
UART PROTOCOL DEFINITION
console to controller:
's'     start toggle
'l'     low power mode toggle
'r'     right turn
'k'     left turn
'z'     zero angle set
'd'     commutation toggle
1       first status frame request
2       second status frame request
3       angle frame request
controller to console:
first status frame:
      1      0
      2      0
      3      transmitter_is_started 
      4      low_power_mode_is_active
      5      direct_antennas_are_connected
      6      un-direct_antennas_are_connected
      7      global_GPS_failure
      8      global_GLONASS_failure
second status frame:
       1      0
       2      1
       3      L1_GPS_amplifier_failure
       4      L2_GPS_amplifier_failure
       5      L1_GLONASS_amplifier_failure
       6      L2_GLONASS_amplifier_failure
       7      GPS_power_failure
       8      GLONASS_power_failure
angle frame
      1    1
      2    LSB of corrected azimuth (360/128 degrees)
      3    360/64 degrees
      4    360/32 degrees
      5    360/16 degrees
      6    360/8 degrees
      7    360/4 degrees
      8    MSB of corrected azimuth (360/2 degrees)
*/

volatile unsigned char flagS,flagZ,flagL,flagR,flagK,flagD,flagU;
volatile unsigned char status[2];
void errorman(void)
{    
     
     if((!(FPIN & IF11)) && (SPIN & IS1))// RFA=0,START=1 only
     {status[1]=status[1]| 0b00100000;}
     else
     {status[1]=status[1]& 0b11011111;}
     
     if((!(FPIN & IF12)) && (SPIN & IS1))
     {status[1]=status[1]| 0b00010000;}
     else
     {status[1]=status[1]& 0b11101111;}
     
     if((!(FPIN & IF21)) && (SPIN & IS1))
     {status[1]=status[1]| 0b00001000;}
     else
     {status[1]=status[1]& 0b11110111;}
     
     if((!(FPIN & IF22)) && (SPIN & IS1))
     {status[1]=status[1]| 0b00100100;}
     else
     {status[1]=status[1]& 0b11011011;}
     
     if(((FPIN & IF10)))//power failure if 0
     {status[1]=status[1]& 0b11111101;}
     else
     {status[1]=status[1]| 0b00000010;}
     
     if(((FPIN & IF20)))
     {status[1]=status[1]& 0b11111110;}
     else
     {status[1]=status[1]| 0b00000001;}
     
     if(((FPIN & IF01)))//integral failure if 0
     {status[0]=status[0]& 0b11111101;}
     else
     {status[0]=status[0]| 0b00000010;}
     
     if(((FPIN & IF02)))
     {status[0]=status[0]& 0b11111110;}
     else
     {status[0]=status[0]|0b00000001;}
}

void startman(void)
{
     if(flagS)
     {flagS=0;
              if((SPIN&IS1)&&(SPIN&IS2))//if both on->off
              {SPORT.OS1=0;}
              else
              {SPORT.OS1=1;}
     }
     else
     {;}
     //delay_us(100);//
     if((SPIN&IS1)&&(SPIN&IS2))
     {status[0]=status[0]|0b00100000;}
     else
     {status[0]=status[0]&0b11011111;}
     
     //failure
     if(SPIN&IS1)
     
     {
     if(SPIN&(1<<OS1));
     else
     { 
     status[0]=status[0]|0b00000010;
     }        

     }
     else 
     {
     if (SPIN&(1<<OS1))
     {
     status[0]=status[0]|0b00000010;
     }
     }
     
     
       if(SPIN&IS2)
     {
     if(SPIN&(1<<OS1));
     else
     { 
     status[0]=status[0]|0b00000001;
     }        

     }
     else 
     {
     if (SPIN&(1<<OS1))
     {
     status[0]=status[0]|0b00000001;
     }
     }
}

void LPMman(void)
{
      if(flagL)
     {flagL=0;
              if((PPIN&IP1)&&(PPIN&IP2))
              {PPORT.OP1=0;}
              else
              {PPORT.OP1=1;}
     }
     else
     {;}
     //delay_us(100);//
     if((PPIN&IP1)&&(PPIN&IP2))
     {status[0]=status[0]|0b00010000;}
     else
     {status[0]=status[0]&0b11101111;}
}

void dirman(void)
{
     if((DPIN&IDD11)||(DPIN&IDD12)||(DPIN&IDD21)||(DPIN&IDD22))
     status[0]=status[0]&0b11110111;
     else
     status[0]=status[0]|0b00001000;
     
     if((DPIN&IDU11)||(DPIN&IDU12)||(DPIN&IDU21)||(DPIN&IDU22))
     status[0]=status[0]&0b11111011;
     else
     status[0]=status[0]|0b00000100;
     if(flagD)
     {flagD=0;
if(status[0]&0b00001000)
     {DPORT.ODD=1;//high to switch
     DPORT.ODU=0;
     DELAYSWITCH
     DPORT.ODD=0;}
     else
     {DPORT.ODU=1;//high to switch
     DPORT.ODD=0;
     DELAYSWITCH
     DPORT.ODU=0;}}
     else
     {;}
}



volatile unsigned char angle0=0, angle=0, setangle=0;
void angleman(void)
{/*status[3]=status[3];*/
         
         //=0;
         EPORT.OE_CSn=0;
         EPORT.OE_CLK=1;
         DELAYSTART
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;;//D9
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D8
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D7
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D6
         DELAYCLK
         if (EPIN & IE_DO)
           angle=angle |(0b00000001);
         else
             angle=angle & (0b11111110);
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D5
         DELAYCLK
         if (EPIN & IE_DO)
           angle=angle |(0b00000010);
         else
             angle=angle & (0b11111101);
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D4
         DELAYCLK
         if (EPIN & IE_DO)
           angle=angle |(0b00000100);
         else
             angle=angle & (0b11111011);
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D3
         DELAYCLK
         if (EPIN & IE_DO)
           angle=angle |(0b00001000);
         else
             angle=angle & (0b11110111);
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D2
         DELAYCLK
         if (EPIN & IE_DO)
           angle=angle |(0b00010000);
         else
             angle=angle & (0b11101111);
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D1
         DELAYCLK
         if (EPIN & IE_DO)
           angle=angle |(0b00100000);
         else
             angle=angle & (0b11011111);
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//D0
         DELAYCLK
         if (EPIN & IE_DO)
           angle=angle |(0b01000000);
         else
             angle=angle & (0b10111111);
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//OCF
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//OOF
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//LIN
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;;//INC
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//DEC
         DELAYCLK
         EPORT.OE_CLK=0;
         DELAYCLK
         EPORT.OE_CLK=1;//PAR
         DELAYCLK
         DELAYCLK
         EPORT.OE_CSn=1;//_EOT
         DELAYSTART
         if (flagZ)
         {flagZ=0;angle0=angle;}
}

void tempman(void)
{
     if(ACSR&(1<<ACO))
     TPORT.OT1=1;
     else
     TPORT.OT1=0;
}

void rotate(void)
{
     if(flagK)
              {flagK=0;setangle++;}
     else
              ;
     if(flagR)
              {flagR=0;setangle--;}
     else
         ;
     if(setangle>angle)
                       {RPORT.ORR=1;RPORT.ORL=0;//high to step
                       DELAYSTEP
                       RPORT.ORR=0;}//step right
     else
     {
         if(setangle<angle)
                           {RPORT.ORL=1;RPORT.ORR=0;
                       DELAYSTEP
                       RPORT.ORL=0;}//step left
         else
             {RPORT.ORR=0;RPORT.ORL=0;}//stop stepping
     }
}
/*
interrupt [USART0_RXC] void ISRUSART(void)
{
            unsigned char rec;
            rec=UDR0;
            switch(rec){
            case 'a':flagS=1;break;
            case 'b':flagL=1;break;
            case 'f':flagR=1;break;
            case 'e':flagK=1;break;
            case 'c':flagD=1;break;
            case 'd':flagZ=1;break;
            case 'g':flagU=1;break;
            case 'h':flagU=2;break;
            case 'i':flagU=3;break;
            default:
            ;
            };
            
            
}  */

void USART_Transmit( unsigned char data )
{
/* Wait for empty transmit buffer */
//while ( !( UCSR0A & (1<<UDRE0)) )
;
/*tranceiver enable*/
PORTE.4=1;
/* Put data into buffer, sends the data */
putchar(data);
/*tranceiver disable*/
PORTE.4=0;
}/*C Atmel Corporation 2011*/

void USARTSEND(void)
{
    unsigned char rec=0b00000000;
    rec=getchar();
            switch(rec){
            case 0b00001001 : flagS=1;break;
            case 0b00001010 : flagL=1;break;
            case 0b01010000 : flagR=1;break;
            case 0b00010100 : flagK=1;break;
            case 0b00101000 : flagD=1;break;
            case 0b10001000 : flagZ=1;break;
            case 0b11000000 : flagU=1;break;
            case 0b00101010 : flagU=2;break;
            case 0b00110010 : flagU=3;break;
            default:
            ;
            };
     switch(flagU){
     case 1: flagU=0;USART_Transmit(status[0]);break;
     case 2: flagU=0;USART_Transmit(status[1]);break;
     case 3: flagU=0;USART_Transmit((angle-angle0));break;
     default: flagU=0;}
     
}



